#include <sys/types.h>
#ifdef WIN32
#include <ctype.h>
#include <memory.h>
#include <string.h>
#include <malloc.h>
#endif
#include "regex.h"

typedef struct {
  struct re_pattern_buffer rpb;
  char fastmap[256];
} rt_thing, *rt;

rt re_initialize_pattern(int case_insensitive)
{
   rt pattern;
   static char case_fold[256];

   re_syntax_options = RE_SYNTAX_POSIX_EXTENDED;

   memset(pattern = (rt) malloc(sizeof(rt_thing)), 0, sizeof(rt_thing));
   pattern->rpb.fastmap = pattern->fastmap;
   pattern->rpb.regs_allocated=REGS_UNALLOCATED;

   if(case_insensitive){
     if(case_fold[255] == 0){
       int i;
       for (i = 0; i < 256; i++) case_fold[i] = i;
       for (i = 'a'; i <= 'z'; i++) case_fold[i] = i - ('a' - 'A');
     }
     pattern->rpb.translate = case_fold;
   } else pattern->rpb.translate = 0;

   return pattern;
}

void re_free_pattern(rt pattern){
  pattern->rpb.fastmap = pattern->rpb.translate = 0;  /* otherwise gnu will attempt to free them */
  regfree(pattern->rpb);
  free(pattern);
}

void re_free_registers(struct re_registers *r){
  if(r->start) free(r->start);
  if(r->end) free(r->end);
  free(r);
}

struct re_registers *re_initialize_registers(){
  struct re_registers *r = (struct re_registers *) malloc(sizeof(struct re_registers));
  memset(r, 0, sizeof(struct re_registers));
  return r;
}

int re_num_registers(struct re_registers *regs)
{
  return (int) regs->num_regs;
}

void re_register(struct re_registers *regs, int index, int *start, int *end)
{
	/* Added cast to int just to remove a warning message */
  if (index < ((int)regs->num_regs)) {
    *start = regs->start[index];
    *end   = regs->end[index];
  }
  else
    *start = *end = 0;
}

typedef void (*fn)();

fn re_trail(){
  return re_free_pattern;
}

#if 0
char *buffer = "DOG";
main() 
{
   char *pattern = (char *) malloc(strlen("(dog|cat)")+1);
   struct re_pattern_buffer *pattern_buffer = re_initialize_pattern(1);

   strcpy(pattern, "(dog|cat)");

   printf("pattern=%s, buffer=%s\n", pattern, buffer);
   printf("compile: %d\n", re_compile_pattern(pattern, strlen(pattern), pattern_buffer));
   printf("match  : %d\n", re_match(pattern_buffer, buffer, strlen(buffer), 0, 0));

}
#endif
